package kr.co.vividnext.sodalive.content.main.tab.asmr

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.AudioContentMainTabRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service
import java.time.DayOfWeek
import java.time.LocalDateTime
import java.time.temporal.TemporalAdjusters

@Service
class AudioContentMainTabAsmrService(
    private val repository: AudioContentMainTabRepository,
    private val bannerService: AudioContentBannerService,
    private val contentRepository: AudioContentRepository,
    private val rankingService: RankingService,
    private val eventService: EventService,
    private val curationRepository: AudioContentCurationQueryRepository
) {
    fun fetchData(member: Member): GetContentMainTabAsmrResponse {
        val isAdult = member.auth != null
        val memberId = member.id!!
        val theme = "ASMR"
        val tabId = 5L

        val contentBannerList = bannerService.getBannerList(
            tabId = tabId,
            memberId = memberId,
            isAdult = isAdult
        )

        val newAsmrContentList = contentRepository.findByTheme(
            memberId = memberId,
            theme = theme,
            isAdult = isAdult,
            contentType = ContentType.ALL,
            limit = 10
        )

        val currentDateTime = LocalDateTime.now()
        val startDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusWeeks(1)
            .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY))
        val endDate = startDate
            .plusDays(6)

        val rankAsmrContentList = rankingService.getContentRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = startDate,
            endDate = endDate,
            theme = theme
        )

        val creatorList = repository.findCreatorByThemeContent(
            memberId = memberId,
            theme = theme,
            minCount = 4
        )

        val salesCountRankContentList = if (creatorList.isNotEmpty()) {
            rankingService.fetchCreatorContentBySalesCountTop4(
                creatorId = creatorList[0].creatorId,
                isAdult = isAdult,
                theme = theme
            )
        } else {
            emptyList()
        }

        val eventBannerList = eventService.getEventList(isAdult = isAdult)

        val curationList = curationRepository.findByContentMainTabId(tabId = tabId, isAdult = isAdult)
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = contentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult
                    )
                )
            }

        return GetContentMainTabAsmrResponse(
            contentBannerList = contentBannerList,
            newAsmrContentList = newAsmrContentList,
            rankAsmrContentList = rankAsmrContentList,
            creatorList = creatorList,
            salesCountRankContentList = salesCountRankContentList,
            eventBannerList = eventBannerList,
            curationList = curationList
        )
    }

    fun getPopularContentByCreator(creatorId: Long, isAdult: Boolean): List<GetAudioContentRankingItem> {
        return rankingService.fetchCreatorContentBySalesCountTop4(
            creatorId = creatorId,
            isAdult = isAdult,
            theme = "ASMR"
        )
    }
}
